@@ -28,8 +28,8 @@ static_cast<ImplClass*>(this)->Visit##CLASS##Decl(            \
                                static_cast<CLASS##Decl*>(D)); \
 break;
 
-#define DEFAULT_DISPATCH(CLASS) void Visit##CLASS##Decl(CLASS##Decl* D) {}
-#define DEFAULT_DISPATCH_VARDECL(CLASS) void Visit##CLASS##Decl(CLASS##Decl* D)\
+#define DEFAULT_DISPATCH(CLASS) void Visit##CLASS##Decl(CLASS##Decl *D) {}
+#define DEFAULT_DISPATCH_VARDECL(CLASS) void Visit##CLASS##Decl(CLASS##Decl *D)\
   { static_cast<ImplClass*>(this)->VisitVarDecl(D); }
 
 
@@ -38,23 +38,23 @@ template <typename ImplClass>
 class CFGRecStmtDeclVisitor : public CFGRecStmtVisitor<ImplClass> {
 public:
 
-  void VisitDeclRefExpr(DeclRefExpr* DR) {
+  void VisitDeclRefExpr(DeclRefExpr *DR) {
     static_cast<ImplClass*>(this)->VisitDecl(DR->getDecl());
   }
 
-  void VisitDeclStmt(DeclStmt* DS) {
+  void VisitDeclStmt(DeclStmt *DS) {
     for (DeclStmt::decl_iterator DI = DS->decl_begin(), DE = DS->decl_end();
         DI != DE; ++DI) {
-      Decl* D = *DI;
+      Decl *D = *DI;
       static_cast<ImplClass*>(this)->VisitDecl(D);
       // Visit the initializer.
-      if (VarDecl* VD = dyn_cast<VarDecl>(D))
-        if (Expr* I = VD->getInit())
+      if (VarDecl *VD = dyn_cast<VarDecl>(D))
+        if (Expr *I = VD->getInit())
           static_cast<ImplClass*>(this)->Visit(I);
     }
   }
 
-  void VisitDecl(Decl* D) {
+  void VisitDecl(Decl *D) {
     switch (D->getKind()) {
         DISPATCH_CASE(Function)
         DISPATCH_CASE(CXXMethod)
@@ -66,10 +66,11 @@ public:
         DISPATCH_CASE(Record)    // FIXME: Refine.  VisitStructDecl?
         DISPATCH_CASE(CXXRecord)
         DISPATCH_CASE(Enum)
+        DISPATCH_CASE(Field)
         DISPATCH_CASE(UsingDirective)
         DISPATCH_CASE(Using)
       default:
-        assert(false && "Subtype of ScopedDecl not handled.");
+        llvm_unreachable("Subtype of ScopedDecl not handled.");
     }
   }
 
@@ -82,8 +83,8 @@ public:
   DEFAULT_DISPATCH(Typedef)
   DEFAULT_DISPATCH(Record)
   DEFAULT_DISPATCH(Enum)
+  DEFAULT_DISPATCH(Field)
   DEFAULT_DISPATCH(ObjCInterface)
-  DEFAULT_DISPATCH(ObjCClass)
   DEFAULT_DISPATCH(ObjCMethod)
   DEFAULT_DISPATCH(ObjCProtocol)
   DEFAULT_DISPATCH(ObjCCategory)
