@@ -10,6 +10,7 @@
 #ifndef CLANG_DRIVER_ARGLIST_H_
 #define CLANG_DRIVER_ARGLIST_H_
 
+#include "clang/Basic/LLVM.h"
 #include "clang/Driver/OptSpecifier.h"
 #include "clang/Driver/Util.h"
 #include "llvm/ADT/SmallVector.h"
@@ -19,12 +20,8 @@
 #include <string>
 #include <vector>
 
-namespace llvm {
-  class Twine;
-}
-
 namespace clang {
-  class Diagnostic;
+  class DiagnosticsEngine;
 
 namespace driver {
   class Arg;
@@ -34,7 +31,7 @@ namespace driver {
   /// arg_iterator - Iterates through arguments stored inside an ArgList.
   class arg_iterator {
     /// The current argument.
-    llvm::SmallVectorImpl<Arg*>::const_iterator Current;
+    SmallVectorImpl<Arg*>::const_iterator Current;
 
     /// The argument list we are iterating over.
     const ArgList &Args;
@@ -58,7 +55,7 @@ namespace driver {
     typedef std::forward_iterator_tag   iterator_category;
     typedef std::ptrdiff_t              difference_type;
 
-    arg_iterator(llvm::SmallVectorImpl<Arg*>::const_iterator it,
+    arg_iterator(SmallVectorImpl<Arg*>::const_iterator it,
                  const ArgList &_Args, OptSpecifier _Id0 = 0U,
                  OptSpecifier _Id1 = 0U, OptSpecifier _Id2 = 0U)
       : Current(it), Args(_Args), Id0(_Id0), Id1(_Id1), Id2(_Id2) {
@@ -101,7 +98,7 @@ namespace driver {
     void operator=(const ArgList &); // DO NOT IMPLEMENT
 
   public:
-    typedef llvm::SmallVector<Arg*, 16> arglist_type;
+    typedef SmallVector<Arg*, 16> arglist_type;
     typedef arglist_type::iterator iterator;
     typedef arglist_type::const_iterator const_iterator;
     typedef arglist_type::reverse_iterator reverse_iterator;
@@ -152,6 +149,13 @@ namespace driver {
       return arg_iterator(Args.end(), *this);
     }
 
+    /// @}
+    /// @name Arg Removal
+    /// @{
+
+    /// eraseArg - Remove any option matching \arg Id.
+    void eraseArg(OptSpecifier Id);
+
     /// @}
     /// @name Arg Access
     /// @{
@@ -181,6 +185,8 @@ namespace driver {
     Arg *getLastArg(OptSpecifier Id0, OptSpecifier Id1, OptSpecifier Id2) const;
     Arg *getLastArg(OptSpecifier Id0, OptSpecifier Id1, OptSpecifier Id2,
                     OptSpecifier Id3) const;
+    Arg *getLastArg(OptSpecifier Id0, OptSpecifier Id1, OptSpecifier Id2,
+                    OptSpecifier Id3, OptSpecifier Id4) const;
 
     /// getArgString - Return the input argument string at \arg Index.
     virtual const char *getArgString(unsigned Index) const = 0;
@@ -195,13 +201,21 @@ namespace driver {
     /// @{
 
     /// getLastArgValue - Return the value of the last argument, or a default.
-    llvm::StringRef getLastArgValue(OptSpecifier Id,
-                                    llvm::StringRef Default = "") const;
+    StringRef getLastArgValue(OptSpecifier Id,
+                                    StringRef Default = "") const;
+
+    /// getLastArgValue - Return the value of the last argument as an integer,
+    /// or a default. If Diags is non-null, emits an error if the argument
+    /// is given, but non-integral.
+    int getLastArgIntValue(OptSpecifier Id, int Default,
+                           DiagnosticsEngine *Diags = 0) const;
 
     /// getLastArgValue - Return the value of the last argument as an integer,
     /// or a default. Emits an error if the argument is given, but non-integral.
     int getLastArgIntValue(OptSpecifier Id, int Default,
-                           Diagnostic &Diags) const;
+                           DiagnosticsEngine &Diags) const {
+      return getLastArgIntValue(Id, Default, &Diags);
+    }
 
     /// getAllArgValues - Get the values of all instances of the given argument
     /// as strings.
@@ -245,25 +259,29 @@ namespace driver {
     /// option id.
     void ClaimAllArgs(OptSpecifier Id0) const;
 
+    /// ClaimAllArgs - Claim all arguments.
+    ///
+    void ClaimAllArgs() const;
+
     /// @}
     /// @name Arg Synthesis
     /// @{
 
     /// MakeArgString - Construct a constant string pointer whose
     /// lifetime will match that of the ArgList.
-    virtual const char *MakeArgString(llvm::StringRef Str) const = 0;
+    virtual const char *MakeArgString(StringRef Str) const = 0;
     const char *MakeArgString(const char *Str) const {
-      return MakeArgString(llvm::StringRef(Str));
+      return MakeArgString(StringRef(Str));
     }
     const char *MakeArgString(std::string Str) const {
-      return MakeArgString(llvm::StringRef(Str));
+      return MakeArgString(StringRef(Str));
     }
-    const char *MakeArgString(const llvm::Twine &Str) const;
+    const char *MakeArgString(const Twine &Str) const;
 
     /// \brief Create an arg string for (\arg LHS + \arg RHS), reusing the
     /// string at \arg Index if possible.
-    const char *GetOrMakeJoinedArgString(unsigned Index, llvm::StringRef LHS,
-                                         llvm::StringRef RHS) const;
+    const char *GetOrMakeJoinedArgString(unsigned Index, StringRef LHS,
+                                         StringRef RHS) const;
 
     /// @}
   };
@@ -304,10 +322,10 @@ namespace driver {
 
   public:
     /// MakeIndex - Get an index for the given string(s).
-    unsigned MakeIndex(llvm::StringRef String0) const;
-    unsigned MakeIndex(llvm::StringRef String0, llvm::StringRef String1) const;
+    unsigned MakeIndex(StringRef String0) const;
+    unsigned MakeIndex(StringRef String0, StringRef String1) const;
 
-    virtual const char *MakeArgString(llvm::StringRef Str) const;
+    virtual const char *MakeArgString(StringRef Str) const;
 
     /// @}
   };
@@ -346,7 +364,7 @@ namespace driver {
       SynthesizedArgs.push_back(A);
     }
 
-    virtual const char *MakeArgString(llvm::StringRef Str) const;
+    virtual const char *MakeArgString(StringRef Str) const;
 
     /// AddFlagArg - Construct a new FlagArg for the given option \arg Id and
     /// append it to the argument list.
@@ -358,7 +376,7 @@ namespace driver {
     /// \arg Id, with the provided \arg Value and append it to the argument
     /// list.
     void AddPositionalArg(const Arg *BaseArg, const Option *Opt,
-                          llvm::StringRef Value) {
+                          StringRef Value) {
       append(MakePositionalArg(BaseArg, Opt, Value));
     }
 
@@ -367,7 +385,7 @@ namespace driver {
     /// \arg Id, with the provided \arg Value and append it to the argument
     /// list.
     void AddSeparateArg(const Arg *BaseArg, const Option *Opt,
-                        llvm::StringRef Value) {
+                        StringRef Value) {
       append(MakeSeparateArg(BaseArg, Opt, Value));
     }
 
@@ -375,7 +393,7 @@ namespace driver {
     /// AddJoinedArg - Construct a new Positional arg for the given option \arg
     /// Id, with the provided \arg Value and append it to the argument list.
     void AddJoinedArg(const Arg *BaseArg, const Option *Opt,
-                      llvm::StringRef Value) {
+                      StringRef Value) {
       append(MakeJoinedArg(BaseArg, Opt, Value));
     }
 
@@ -387,17 +405,17 @@ namespace driver {
     /// MakePositionalArg - Construct a new Positional arg for the
     /// given option \arg Id, with the provided \arg Value.
     Arg *MakePositionalArg(const Arg *BaseArg, const Option *Opt,
-                           llvm::StringRef Value) const;
+                           StringRef Value) const;
 
     /// MakeSeparateArg - Construct a new Positional arg for the
     /// given option \arg Id, with the provided \arg Value.
     Arg *MakeSeparateArg(const Arg *BaseArg, const Option *Opt,
-                         llvm::StringRef Value) const;
+                         StringRef Value) const;
 
     /// MakeJoinedArg - Construct a new Positional arg for the
     /// given option \arg Id, with the provided \arg Value.
     Arg *MakeJoinedArg(const Arg *BaseArg, const Option *Opt,
-                       llvm::StringRef Value) const;
+                       StringRef Value) const;
 
     /// @}
   };
