@@ -25,18 +25,21 @@
 /* Installation directory for data files */
 /* #undef LLVM_DATADIR */
 
+/* Target triple LLVM will generate code for by default */
+#define LLVM_DEFAULT_TARGET_TRIPLE "sparc64-unknown-openbsd5.2"
+
 /* Installation directory for documentation */
 /* #undef LLVM_DOCSDIR */
 
+/* Define if threads enabled */
+#define LLVM_ENABLE_THREADS 1
+
 /* Installation directory for config files */
 /* #undef LLVM_ETCDIR */
 
 /* Has gcc/MSVC atomic intrinsics */
 #define LLVM_HAS_ATOMICS 1
 
-/* Host triple we were built on */
-#define LLVM_HOSTTRIPLE "powerpc-unknown-openbsd5.1"
-
 /* Installation directory for include files */
 /* #undef LLVM_INCLUDEDIR */
 
@@ -50,22 +53,25 @@
 /* #undef LLVM_MANDIR */
 
 /* LLVM architecture name for the native architecture, if available */
-#define LLVM_NATIVE_ARCH PowerPC
+#define LLVM_NATIVE_ARCH Sparc
 
 /* LLVM name for the native AsmParser init function, if available */
 /* #undef LLVM_NATIVE_ASMPARSER */
 
 /* LLVM name for the native AsmPrinter init function, if available */
-#define LLVM_NATIVE_ASMPRINTER LLVMInitializePowerPCAsmPrinter
+#define LLVM_NATIVE_ASMPRINTER LLVMInitializeSparcAsmPrinter
+
+/* LLVM name for the native Disassembler init function, if available */
+/* #undef LLVM_NATIVE_DISASSEMBLER */
 
 /* LLVM name for the native Target init function, if available */
-#define LLVM_NATIVE_TARGET LLVMInitializePowerPCTarget
+#define LLVM_NATIVE_TARGET LLVMInitializeSparcTarget
 
 /* LLVM name for the native TargetInfo init function, if available */
-#define LLVM_NATIVE_TARGETINFO LLVMInitializePowerPCTargetInfo
+#define LLVM_NATIVE_TARGETINFO LLVMInitializeSparcTargetInfo
 
 /* LLVM name for the native target MC init function, if available */
-#define LLVM_NATIVE_TARGETMC LLVMInitializePowerPCTargetMC
+#define LLVM_NATIVE_TARGETMC LLVMInitializeSparcTargetMC
 
 /* Define if this is Unixish platform */
 #define LLVM_ON_UNIX 1
@@ -103,4 +109,10 @@
 /* Installation prefix directory */
 #define LLVM_PREFIX "/usr/local"
 
+/* Major version of the LLVM API */
+#define LLVM_VERSION_MAJOR 3
+
+/* Minor version of the LLVM API */
+#define LLVM_VERSION_MINOR 1
+
 #endif
